header
{
    package com.unitesk.java5;
    
    import java.io.File;
    
    import com.unitesk.atp.messages.MessageReceiverAdapter;
    import com.unitesk.atp.text.location.Position;
}

class Java5Lexer extends Lexer;
options
{
    k = 4;
    testLiterals = false;
    charVocabulary = '\u0000'..'\uFFFF';
    defaultErrorHandler = true;
}

tokens
{
    // see NUMERIC_LITERAL
    // values are used only for token name, should not be matched by identifier rule
    DOT = ".";      
    VARARG = "...";
    INTEGER_LITERAL = "an integer literal";
    FLOATING_POINT_LITERAL = "a floating point literal";

// Java keywords
    // Packages and importing
    IMPORT        = "import";
    PACKAGE       = "package";

    // Reference types
    CLASS         = "class";
    ENUM          = "enum";
    INTERFACE     = "interface";

    // Inheritance
    EXTENDS      = "extends";
    IMPLEMENTS   = "implements";
    
    // Modifiers
    ABSTRACT     = "abstract";    
    CONST        = "const";    
    FINAL        = "final";
    NATIVE       = "native";
    PRIVATE      = "private"; 
    PROTECTED    = "protected";    
    PUBLIC       = "public";
    STATIC       = "static";
    STRICTFP     = "strictfp";
    SYNCHRONIZED = "synchronized";
    TRANSIENT    = "transient";
    VOLATILE     = "volatile";

    // Accessing
    SUPER        = "super";
    THIS         = "this";

	// Built-in types
    BOOLEAN       = "boolean";
    BYTE          = "byte";
    CHAR          = "char";
    DOUBLE        = "double";
    FLOAT         = "float";
    INT           = "int"; 
    LONG          = "long";
    SHORT         = "short"; 
    // Empty result type
    VOID          = "void";
    // Boolean literals
    TRUE          = "true";
    FALSE         = "false";
    // Null literal
    NULL          = "null";

    // Statements
    ASSERT       = "assert";
    BREAK        = "break";
    CASE         = "case";
    CATCH        = "catch";
    CONTINUE     = "continue";
    DEFAULT      = "default";
    DO           = "do";
    ELSE         = "else";
    FINALLY      = "finally";
    FOR          = "for";
    GOTO         = "goto";
    IF           = "if";
    INSTANCEOF   = "instanceof";
    NEW          = "new";    
    RETURN       = "return";
    SWITCH       = "switch";
    THROW        = "throw";
    TRY          = "try";
    WHILE        = "while";

    // Throws-clause
    THROWS       = "throws";
}

{
	protected int version = 5;
	
	/**
	 * Set Java version.
	 * Allowed values are 3, 4, 5 that corresponds to 1.3, 1.4 and 1.5.
	 */
    public void setVersion( int version )
    {
    	this.version = version;
    
    	if( version < 4 )
        {    	
    		literals.remove( new ANTLRHashString("assert", this) );
    	}
    }

    /**
     * Support for new keywords.
     */
	public void addLiteral( String str, int code )
	{
		literals.put( new ANTLRHashString(str, this), new Integer( code ) );
	}

	public void recover( RecognitionException ex
	                   , BitSet tokenSet
	                   ) throws CharStreamException 
	{
		consume();
		consumeUntil( tokenSet );
	}
	
    protected File file;
    protected MessageReceiverAdapter mbox;
    
    public void setFile( File file )
    {
        this.file = file;
    }    
    
    public void setMessageBox( MessageReceiverAdapter mbox )
    {
    	this.mbox = mbox;
    }
    
    public void reportError( RecognitionException ex )
    {
        String message = ex.getMessage();
        message = message.replace( "'\\r'", "EOL" );
        message = message.replace( "'\\n'", "EOL" );
        mbox.error( file
                  , new Position( ex.getLine()
                                , ex.getColumn()
                                )
                  , new Message.SyntaxError( message )
                  );
    }
}

CTRL_Z options { paraphrase = "Ctrl-Z"; } : '\u001a';

// PUNCTUATORS
COMMA      options { paraphrase = "','"; } : ',' ;
SEMICOLON  options { paraphrase = "';'"; } : ';' ;
COLON      options { paraphrase = "':'"; } : ':' ;

// SEPARATORS
LBRACE   options { paraphrase = "'{'"; } : '{' ;
RBRACE   options { paraphrase = "'}'"; } : '}' ;
LBRACKET options { paraphrase = "'['"; } : '[' ;
RBRACKET options { paraphrase = "']'"; } : ']' ;
LPARENT  options { paraphrase = "'('"; } : '(' ;
RPARENT  options { paraphrase = "')'"; } : ')' ;

// OPERATORS
AT            options { paraphrase = "'@'"; }    : "@"   ;
BIT_OR        options { paraphrase = "'|'"; }    : "|"   ;
PLUS          options { paraphrase = "'+'"; }    : "+"   ;
MINUS         options { paraphrase = "'-'"; }    : "-"   ;
MULT          options { paraphrase = "'*'"; }    : "*"   ;
DIVIDE        options { paraphrase = "'/'"; }    : "/"   ;
REM           options { paraphrase = "'%'"; }    : "%"   ;
BIT_AND       options { paraphrase = "'&'"; }    : "&"   ;
XOR           options { paraphrase = "'^'"; }    : "^"   ;
NOT           options { paraphrase = "'!'"; }    : "!"   ;
COMPL         options { paraphrase = "'~'"; }    : "~"   ;
ASSIGN        options { paraphrase = "'='"; }    : "="   ;
LTHN          options { paraphrase = "'<'"; }    : "<"   ;
GTHN          options { paraphrase = "'>'"; }    : ">"   ;
QUESTION      options { paraphrase = "'?'"; }    : "?"   ;
INCR          options { paraphrase = "'++'"; }   : "++"  ;     
DECR          options { paraphrase = "'--'"; }   : "--"  ;
SC_AND        options { paraphrase = "'&&'"; }   : "&&"  ;
SC_OR         options { paraphrase = "'||'"; }   : "||"  ;
LSHIFT        options { paraphrase = "'<<'"; }   : "<<"  ;
RSHIFT        options { paraphrase = "'>>'"; }   : ">>"  ;
URSHIFT       options { paraphrase = "'>>>'"; }  : ">>>" ;
EQ            options { paraphrase = "'=='"; }   : "=="  ;
NE            options { paraphrase = "'!='"; }   : "!="  ;
LE            options { paraphrase = "'<='"; }   : "<="  ;
GE            options { paraphrase = "'>='"; }   : ">="  ;
PLUS_ASSIGN   options { paraphrase = "'+='"; }   : "+="  ;
MINUS_ASSIGN  options { paraphrase = "'-='"; }   : "-="  ;
MULT_ASSIGN   options { paraphrase = "'*='"; }   : "*="  ;
DIVIDE_ASSIGN options { paraphrase = "'/='"; }   : "/="  ;
REM_ASSIGN    options { paraphrase = "'%='"; }   : "%="  ;
AND_ASSIGN    options { paraphrase = "'&='"; }   : "&="  ;
OR_ASSIGN     options { paraphrase = "'|='"; }   : "|="  ;
XOR_ASSIGN    options { paraphrase = "'^='"; }   : "^="  ;
LSHIFT_ASSIGN options { paraphrase = "'<<='"; }  : "<<=" ;
RSHIFT_ASSIGN options { paraphrase = "'>>='"; }  : ">>=" ;
URSHIFT_ASSIGN options { paraphrase = "'>>>='"; } : ">>>=";
// not used by Java itself, declared for language extentions 
SHARP         options { paraphrase = "'#'"; }    : "#";

protected
NEW_LINE
:  
    ( options { generateAmbigWarnings = false; }:
        '\r' '\n'  
      | '\r'
      | '\n' 
    )
    { newline(); }
;

WHITESPACE
options { paraphrase = "a whitespace"; }
: 
    ( options { greedy = true; } :
      ' ' | '\t' | '\f' | NEW_LINE
    )+
;

SINGLE_LINE_COMMENT 
options { paraphrase = "a comment"; }
:
    "//" ( ~( '\n' |'\r' | '\uFFFF' ) )* 
    // single line comment may not be followed by new line at the end of file
    ( NEW_LINE )?
;

MULTI_LINE_COMMENT
options { paraphrase = "a comment"; }
:
    "/*"
    (  NEW_LINE
     | { LA(2) != '/' }?  '*' 
     | ~( '\n' | '\r' | '*' | '\uFFFF' )
    )* 
    "*/" 
;

IDENTIFIER
options 
{ 
	paraphrase = "an identifier"; 
	testLiterals = true;
}
:
    ID_START ( ID_PART )*
;

protected ID_START
:
    '_' 
  | '$' 
  | { Character.isJavaIdentifierStart( LA(1) ) }? 
    (            
      ~(
           '_' | '$' | '/' | '*' | '0'..'9' 
         | '.' | '\'' | '\\' | '"' | '\t' | '\n' 
         | '\r' | ' ' | '\f' | '(' | ')'
         | '{' | '}' | '[' | ']'| ';' | ',' | '='
         | '+' | '~' | '&' | '<' | '>' | '-' | '!'
         | '^' | '%' | ':' | '?' | '|'| '@' | '\uFFFF' | '\u001A'
         | '#'
       )
    )
;

protected ID_PART 
:
      '_' 
    | '$' 
    | {Character.isJavaIdentifierPart(LA(1))}?
      (
        ~(
              '_' | '$' | '/' | '*'
            | '.' | '\'' | '\\' | '"' | '\t' | '\n'
            | '\r' | ' ' | '\f' | '(' | ')'
            | '{' | '}' | '[' | ']'| ';' | ',' | '='
            | '+' | '~' | '&' | '<' | '>' | '-' | '!'
            | '^' | '%' | ':' | '?' | '|' | '@' | '\uFFFF'
            | '#'
        )
      )
;

// Literals

NUMERIC_LITERAL
{ 
  int numOctal = 0;
  boolean isFloat = false; 
}
:
    '.' { $setType( DOT ); } // Simple "."
    (   // Float number begining with "."
        ( DIGIT )+ 
        ( EXPONENT_PART )? 
        ( FLOATING_POINT_SUFFIX )? 
        { $setType( FLOATING_POINT_LITERAL ); }
      | ".." { $setType( VARARG ); } // JDK 1.5 token for variable length arguments
    )?
  | '0' (   ( 'x' | 'X' ) (   ( HEXDIGIT )+ (   // HexadecimalFloatingPointLiteral
                                                ( '.' ( HEXDIGIT )* )? BINARY_EXPONENT ( FLOATING_POINT_SUFFIX )? { $setType( FLOATING_POINT_LITERAL ); }
                                                // HexIntegerLiteral
                                              | ( INTEGER_SUFFIX )? { $setType( INTEGER_LITERAL ); }
                                            )
                            | // HexadecimalFloatingPointLiteral
                              '.' ( HEXDIGIT )+ BINARY_EXPONENT ( FLOATING_POINT_SUFFIX )? { $setType( FLOATING_POINT_LITERAL ); }
                          )
          | // It's alternative implementation of the commented code below
            (   OCTDIGIT { numOctal = numOctal == -1 ? -1 : numOctal+1; } 
              | '8'..'9' { numOctal = -1; }
            )*  
            (   ( '.' ( DIGIT )* { isFloat = true; } )?
                ( EXPONENT_PART { isFloat = true; } )?
                ( FLOATING_POINT_SUFFIX { isFloat = true; } )?
                { if( isFloat ) $setType( FLOATING_POINT_LITERAL ); else $setType( INTEGER_LITERAL ); }
              | { numOctal >= 0 }? INTEGER_SUFFIX { $setType( INTEGER_LITERAL ); }               
            )
                          
//          | // DecimalFloatingPointLiteral
//            ( ( DIGIT )* ( '.' | 'e' | 'E' | FLOATING_POINT_SUFFIX ) )=> 
//            ( DIGIT )* ( '.' ( DIGIT )* )? ( EXPONENT_PART )? ( FLOATING_POINT_SUFFIX )? { $setType( FLOATING_POINT_LITERAL ); }
//          | // OctalIntegerLiteral
//            ( OCTDIGIT )+ ( INTEGER_SUFFIX )? { $setType( INTEGER_LITERAL ); }
//          | // Zero
//            ( INTEGER_SUFFIX )? { $setType( INTEGER_LITERAL ); }
        )
  | NON_ZERO_DIGIT ( DIGIT )*
    (   // DecimalFloatingPointLiteral
        ( '.' | 'e' | 'E' | FLOATING_POINT_SUFFIX ) =>
        ( '.' ( DIGIT )* )? ( EXPONENT_PART )? ( FLOATING_POINT_SUFFIX )? { $setType( FLOATING_POINT_LITERAL ); }
      | // DecimalIntegerLiteral
        ( INTEGER_SUFFIX )? { $setType( INTEGER_LITERAL ); }
    )
;

protected
DECIMAL_INTEGER_LITERAL :
    ( DIGIT )+ (INTEGER_SUFFIX )?
;

protected
NON_ZERO_DIGIT : '1'..'9' ;

protected
DIGIT : '0'..'9' ;

protected
OCTDIGIT : '0'..'7' ;

protected
INTEGER_SUFFIX : 'l' | 'L' ;

protected
HEXDIGIT
:
      DIGIT
    | 'A'..'F' 
    | 'a'..'f'
;

protected
BINARY_EXPONENT
: 
    ( 'p' | 'P' )  ( '+' | '-' )? ( DIGIT )+
;
protected
EXPONENT_PART
: 
    ( 'e' | 'E' )  ( '+' | '-' )? ( DIGIT )+
;

protected
FLOATING_POINT_SUFFIX 
:
    'F' | 'f' | 'D' | 'd'
;

CHARACTER_LITERAL
options { paraphrase = "a character literal"; }
: 
    '\''
    (   ( ( SINGLE_CHARACTER | ESCAPE_SEQUENCE ) '\'' ) =>
          ( SINGLE_CHARACTER | ESCAPE_SEQUENCE ) '\''    	    
      | '\''
	    {
	        mbox.error( file
	                  , new Position( getLine()
	                                , getColumn()-2
	                                )
	                  , new Message.SyntaxError( "empty character literal" )
	                  );
	    }
	  | {
            mbox.error( file
                      , new Position( getLine()
                                    , getColumn()
                                    )
                      , new Message.SyntaxError( "unclosed character literal" )
                      );
        }
    )
;

protected 
SINGLE_CHARACTER 
: 
    ~( '\'' | '\\' | '\r' | '\n' )
;

protected 
ESCAPE_SEQUENCE 
	:	'\\'
		(	'n'
		|	'r'
		|	't'
		|	'b'
		|	'f'
		|	'"'
		|	'\''
		|	'\\'
		|	'0'..'3'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
				(
					options {
						warnWhenFollowAmbig = false;
					}
				:	'0'..'7'
				)?
			)?
		|	'4'..'7'
			(
				options {
					warnWhenFollowAmbig = false;
				}
			:	'0'..'7'
			)?
		)
	;

STRING_LITERAL
options { paraphrase = "a string literal"; }:
    '"'
    (   ~( '\\' | '"' | '\r' | '\n' )
      | ESCAPE_SEQUENCE
    )*
    '"'
;
