/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

import jatva.iterators.AbstractIterator;

public class BooleanObjectIterator
  extends    AbstractIterator
  implements jatva.iterators.Iterator
{
  public static Boolean[] defaultValues = {
     null
   , new Boolean(true)
   , new Boolean(false)
   , new Boolean(true)
   , null
  };
  static
  {
    defaultValues[4] = defaultValues[1];
  }

  protected Boolean[] values = defaultValues;
  protected int index = 0;
  
  public BooleanObjectIterator()
  {
  }
  
  public BooleanObjectIterator(Boolean[] values)
  {
    if(values != null) this.values = values;
    else               this.values = new Boolean[0]; 
  }
  
  public BooleanObjectIterator(BooleanObjectIterator r)
  {
    if(r != null)
    {
      index  = r.index;
      
      values = new Boolean[r.values.length];
      
      for(int i = 0; i < values.length; i++)
        values[i] = (r.values[i] == null)?null:(new Boolean(r.values[i].booleanValue()));
    }
    else this.values = new Boolean[0]; 
  }

  public Object clone()
  {
    return new BooleanObjectIterator(this);
  }
  
  public boolean equals(Object o)
  {
    if(!(o instanceof BooleanObjectIterator))
      return false;

    BooleanObjectIterator obj = (BooleanObjectIterator)o;

    if(index != obj.index) return false;
    if(values.length != obj.values.length) return false;
    
    for(int i = 0; i < values.length; i++)
      if(   values[i] == null && obj.values[i] != null
         || values[i] != null && !values[i].equals(obj.values[i])
        ) return false;
    
    return true;   
  }
  
  public Boolean value()
  {
    return values[index];
  }
  
  public boolean stop()
  {
    return index >= values.length;
  }
  
  public void shift()
  {
    index++;
  }

  public String toString()
  {
    if(stop()) return jatva.iterators.Iterator.EXHAUSTED;
    return "" + value();
  }
  
  public int hashCode()
  {
    int r = 113*index;
    for(int i = 0; i < values.length; i++)
    { 
      r *= 3;
      r += (values[i] == null)?0:values[i].hashCode();   
    }
    
    return r;
  }

  public Object next()
  {
    Object r = value();
    shift();
    return r;
  }
}
