/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.iterators.lang;

public class ObjectIterator
  extends    AbstractObjectIterator
  implements IObjectIterator
{
  public static Object[] defaultValues = {
     null
   , new Object()
   , new Character('G')
   , new Integer(Integer.MIN_VALUE - 1)
   , null
   , new Character('o')
   , null
   , "Hello world"
   , new java.util.Vector(18, 15)
   , null
  };
  static
  {
    defaultValues[4] = defaultValues[2];
    
    try
    {
     defaultValues[6] = Class.forName("java.lang.Object");
    }
    catch(ClassNotFoundException ex)
    {
      ex.printStackTrace();
    }
  }
  
  protected Object[] values = defaultValues;
  protected int index = 0;
  
  public ObjectIterator()
  {
  }
  
  public ObjectIterator(Object[] values)
  {
    if(values == null) throw new IllegalArgumentException("Argument should not be null");
    this.values = values;
  }
  
  public ObjectIterator(ObjectIterator right)
  {
    index = right.index;
    values = right.values;
  }
  
  public void addObject(Object n)
  {
    Object[] newValues = new Object[values.length+1];
    System.arraycopy(values, 0, newValues, 0, values.length);
    newValues[values.length] = n;
    values = newValues;
  }

  public synchronized Object clone()
  {
    return new ObjectIterator(this);
  }
  
  public synchronized void shift()
  {
    index++;
  }
  
  public Object value()
  {
    return values[index];
  }
  
  public boolean stop()
  {
    return index >= values.length;
  }

  public boolean equals(Object o) 
  {
    if(!(o instanceof ObjectIterator))
      return false;

    ObjectIterator obj = (ObjectIterator)o;

    if(index != obj.index) return false;
    if(values.length != obj.values.length) return false;
    
    for(int i = 0; i < values.length; i++)
      if(   values[i] == null && obj.values[i] != null
         || values[i] != null && !values[i].equals(obj.values[i])
        ) return false;
    
    return true;   
  }

  public int hashCode ()
  {
    int r = index;
    for(int i = 0; i < values.length; i++)
      r += values[i].hashCode()*i;
        
    return r;
  }
}
