/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang;

import jatva.util.Utils;

public class InvariantException extends StateSynchronizationException
{
    private static final String violationInfo = "Invariant violation";
    private static final String exceptionInfo = "Exception in invariant";

    public InvariantException( String p_class, String invariant )
    {
        super( violationInfo
             , new String[]{ "class", "invariant" }
             , new String[]{ p_class, invariant }
             );
    }

    public InvariantException( String p_class, String invariant, String comment )
    {
        super( Utils.appendCommentToFailure( violationInfo, comment )
             , new String[]{ "class", "invariant" }
             , new String[]{ p_class, invariant }
             );
    }

    public InvariantException( ModelObject object, String invariant )
    {
        super( violationInfo
             , new String[]{ "object", "invariant" }
             , new String[]{ formatModelObject( object ), invariant }
             );
    }

    public InvariantException( ModelObject object, String invariant, String comment )
    {
        super( Utils.appendCommentToFailure( violationInfo, comment )
             , new String[]{ "object", "invariant" }
             , new String[]{ formatModelObject( object ), invariant }
             );
    }

    public InvariantException( String p_class, String invariant, Throwable error )
    {
        super( exceptionInfo
             , error
             , new String[]{ "class", "invariant" }
             , new String[]{ p_class, invariant }
             );
    }

    public InvariantException( ModelObject object, String invariant, Throwable error )
    {
        super( exceptionInfo
             , error
             , new String[]{ "object", "invariant" }
             , new String[]{ formatModelObject( object ), invariant }
             );
    }
}
