/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.lang;

import java.util.Properties;

/**
 * This class respresents runtime test properties.
 * For now all properties are considered as boolean values.
 * A test property is set if the corresponding system property is 
 * explicitly set and its value is string "true".
 * Otherwise a test property is false.
 * Each property has a name that is used as a key in system properties.
 * It is desirable for names of all JavaTESK properies to start with "javatesk." prefix.
 * @author almo
 */
public enum TestProperty
{
    /**
     * If this property is set then runtime 
     * control of tautologies evaualtion is omitted.
     */
    NoTautologiesCheck( "javatesk.tautologies.nocheck" );

    TestProperty( String name )
    { this.name = name; }

    
    private boolean _isSet;
    private final String name;

    public String getName(){ return name; }

    public boolean isSet(){ return _isSet; }
    public void set(){ _isSet = true; }
    public void unset(){ _isSet = false; }

    /**
     * Reads properties from 'System.getProperties()' collection.
     */
    public static void loadProperties()
    {
        clearProperties();

        Properties sys_p = System.getProperties();
        for( TestProperty tp : TestProperty.values() )
        {
            Object o = sys_p.get( tp.getName() );

            if( o == null )
                continue;
            
            if( !(o instanceof String) )
                continue;

            String s = (String)o;

            if( !(s.equals( "true" )) )
                continue;

            tp.set();
        }
    }
    
    private static void clearProperties()
    {
        for( TestProperty tp : TestProperty.values() )
            tp.unset();
    }
}
