/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.util;

import jatva.lang.InsufficientAccessibilityError;
import jatva.lang.TranslationError;

import java.lang.reflect.Constructor;

public class CtorInfo extends OperationInfo
{
    private Constructor<?> ctor;

    public CtorInfo( String extendedReflectSignature )
    {
        super( extendedReflectSignature );
    }

    @Override
    protected final Class<?>[] obtainExecutableMember()
    {
        ctor = Utils.getConstructorByPartialSignature( reflectSignature );

        if( ctor == null )
            throw new TranslationError( "Invalid constructor signature"
                                      , new String[]{ "constructor signature" }
                                      , new String[]{ reflectSignature }
                                      );

        try
        {
          ctor.setAccessible( true );
        }
        catch( SecurityException e )
        {
          throw new InsufficientAccessibilityError( "Insufficient rights to call implementation constructor through reflection"
                                                  , e
                                                  , new String[]{"constructor"}
                                                  , new String[]{reflectSignature}
                                                  );
        }

        this.declaringClass = Utils.prettifyClass( ctor.getDeclaringClass() );
        this.returnType = declaringClass;

        return ctor.getParameterTypes();
    }

    public Constructor<?> getCtor()
    {
        return ctor;
    }

    @Override
    protected void buildPrettySignature()
    {
        StringBuffer res = new StringBuffer();
        res.append( declaringClass );
        appendParTypes( res );
        this.prettySignature = res.toString();
    }
}
