/*
 * Copyright 2005-2009 ISP RAS (http://www.ispras.ru), UniTESK Lab (http://www.unitesk.com)
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
package jatva.util.log;

/**
 * Synchronized log able to print an array of string atomically.
 *
 * Each block of strings is prefaced with a header and finished with a footer.
 * Ancestor classes are meant to use some identification of the thread executing
 * the printing in the header (or footer).
 * @author <a href="mailto:monakhov@ispras.ru">Alexander Monakhov</a>
 */
public abstract class ThreadLog
{
    protected abstract void printHeader();

    protected abstract void printLine( String l );

    protected abstract void printFooter();

    /**
     * Prints all the strings individually, overall string
     * output is prefaced with {@link #printHeader() header} and
     * closed with {@link #printFooter() footer}.
     *
     * Each string is splitted further with new line character '\n'.
     */
    public synchronized void print( String ... texts )
    {
        printHeader();
        for( String t : texts )
        {
            String[] lines = t.split( "\n" );
            for( String l : lines )
            {
                printLine( l );
            }
        }
        printFooter();
    }
}
